@@ -62,8 +62,9 @@ enum
   PROP_ME_METHOD,
   PROP_BUFSIZE,
   PROP_RTP_PAYLOAD_SIZE,
-  PROP_CFG_BASE,
+  PROP_MAX_THREADS,
   PROP_COMPLIANCE,
+  PROP_CFG_BASE,
 };
 
 #define GST_TYPE_ME_METHOD (gst_ffmpegvidenc_me_method_get_type())
@@ -174,6 +175,7 @@ gst_ffmpegvidenc_class_init (GstFFMpegVidEncClass * klass)
 {
   GObjectClass *gobject_class;
   GstVideoEncoderClass *venc_class;
+  int caps;
 
   gobject_class = (GObjectClass *) klass;
   venc_class = (GstVideoEncoderClass *) klass;
@@ -207,6 +209,14 @@ gst_ffmpegvidenc_class_init (GstFFMpegVidEncClass * klass)
           "RTP Payload Size", "Target GOB length", 0, G_MAXINT, 0,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  caps = klass->in_plugin->capabilities;
+  if (caps & (CODEC_CAP_FRAME_THREADS | CODEC_CAP_SLICE_THREADS)) {
+    g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MAX_THREADS,
+        g_param_spec_int ("max-threads", "Maximum encode threads",
+            "Maximum number of worker threads to spawn. (0 = auto)",
+            0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  }
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_COMPLIANCE,
       g_param_spec_enum ("compliance", "Compliance",
           "Adherence of the encoder to the specifications",
@@ -214,7 +224,7 @@ gst_ffmpegvidenc_class_init (GstFFMpegVidEncClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* register additional properties, possibly dependent on the exact CODEC */
-  gst_ffmpeg_cfg_install_property (klass, PROP_COMPLIANCE);
+  gst_ffmpeg_cfg_install_property (klass, PROP_CFG_BASE);
 
   venc_class->start = gst_ffmpegvidenc_start;
   venc_class->stop = gst_ffmpegvidenc_stop;
@@ -247,6 +257,7 @@ gst_ffmpegvidenc_init (GstFFMpegVidEnc * ffmpegenc)
   ffmpegenc->gop_size = DEFAULT_VIDEO_GOP_SIZE;
   ffmpegenc->rtp_payload_size = 0;
   ffmpegenc->compliance = FFMPEG_DEFAULT_COMPLIANCE;
+  ffmpegenc->max_threads = 0;
 
   ffmpegenc->lmin = 2;
   ffmpegenc->lmax = 31;
@@ -263,6 +274,7 @@ gst_ffmpegvidenc_finalize (GObject * object)
   gst_ffmpeg_cfg_finalize (ffmpegenc);
 
   /* clean up remaining allocated data */
+  gst_ffmpeg_avcodec_close (ffmpegenc->context);
   av_free (ffmpegenc->context);
   avcodec_free_frame (&ffmpegenc->picture);
 
@@ -319,6 +331,14 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   GST_DEBUG_OBJECT (ffmpegenc, "Setting avcontext to bitrate %d, gop_size %d",
       ffmpegenc->bitrate, ffmpegenc->gop_size);
 
+  if (ffmpegenc->max_threads == 0) {
+    if (!(oclass->in_plugin->capabilities & CODEC_CAP_AUTO_THREADS))
+      ffmpegenc->context->thread_count = gst_ffmpeg_auto_max_threads ();
+    else
+      ffmpegenc->context->thread_count = 0;
+  } else
+    ffmpegenc->context->thread_count = ffmpegenc->max_threads;
+
   /* RTP payload used for GOB production (for Asterisk) */
   if (ffmpegenc->rtp_payload_size) {
     ffmpegenc->context->rtp_payload_size = ffmpegenc->rtp_payload_size;
@@ -346,7 +366,6 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   ffmpegenc->context->coder_type = 0;
   ffmpegenc->context->context_model = 0;
   ffmpegenc->context->scenechange_threshold = 0;
-  ffmpegenc->context->inter_threshold = 0;
 
   /* and last but not least the pass; CBR, 2-pass, etc */
   ffmpegenc->context->flags |= ffmpegenc->pass;
@@ -381,6 +400,11 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   /* fetch pix_fmt, fps, par, width, height... */
   gst_ffmpeg_videoinfo_to_context (&state->info, ffmpegenc->context);
 
+  /* sanitize time base */
+  if (ffmpegenc->context->time_base.num <= 0
+      || ffmpegenc->context->time_base.den <= 0)
+    goto insane_timebase;
+
   if ((oclass->in_plugin->id == AV_CODEC_ID_MPEG4)
       && (ffmpegenc->context->time_base.den > 65535)) {
     /* MPEG4 Standards do not support time_base denominator greater than
@@ -409,24 +433,6 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
         : ffmpegenc->max_key_interval;
   }
 
-  /* open codec */
-  if (gst_ffmpeg_avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0)
-    goto open_codec_fail;
-
-  /* second pass stats buffer no longer needed */
-  if (ffmpegenc->context->stats_in)
-    g_free (ffmpegenc->context->stats_in);
-
-  /* is the colourspace correct? */
-  if (pix_fmt != ffmpegenc->context->pix_fmt)
-    goto pix_fmt_err;
-
-  /* we may have failed mapping caps to a pixfmt,
-   * and quite some codecs do not make up their own mind about that
-   * in any case, _NONE can never work out later on */
-  if (pix_fmt == PIX_FMT_NONE)
-    goto bad_input_fmt;
-
   /* some codecs support more than one format, first auto-choose one */
   GST_DEBUG_OBJECT (ffmpegenc, "picking an output format ...");
   allowed_caps = gst_pad_get_allowed_caps (GST_VIDEO_ENCODER_SRC_PAD (encoder));
@@ -455,9 +461,29 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   gst_caps_unref (other_caps);
   if (gst_caps_is_empty (icaps)) {
     gst_caps_unref (icaps);
-    return FALSE;
+    goto unsupported_codec;
   }
-  icaps = gst_caps_truncate (icaps);
+  icaps = gst_caps_fixate (icaps);
+
+  GST_DEBUG_OBJECT (ffmpegenc, "codec flags 0x%08x", ffmpegenc->context->flags);
+
+  /* open codec */
+  if (gst_ffmpeg_avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0)
+    goto open_codec_fail;
+
+  /* is the colourspace correct? */
+  if (pix_fmt != ffmpegenc->context->pix_fmt)
+    goto pix_fmt_err;
+
+  /* we may have failed mapping caps to a pixfmt,
+   * and quite some codecs do not make up their own mind about that
+   * in any case, _NONE can never work out later on */
+  if (pix_fmt == PIX_FMT_NONE)
+    goto bad_input_fmt;
+
+  /* second pass stats buffer no longer needed */
+  if (ffmpegenc->context->stats_in)
+    g_free (ffmpegenc->context->stats_in);
 
   /* Store input state and set output state */
   if (ffmpegenc->input_state)
@@ -488,45 +514,50 @@ file_read_err:
     return FALSE;
   }
 
+insane_timebase:
+  {
+    GST_ERROR_OBJECT (ffmpegenc, "Rejecting time base %d/%d",
+        ffmpegenc->context->time_base.den, ffmpegenc->context->time_base.num);
+    goto cleanup_stats_in;
+  }
+unsupported_codec:
+  {
+    GST_DEBUG ("Unsupported codec - no caps found");
+    goto cleanup_stats_in;
+  }
 open_codec_fail:
   {
-    gst_ffmpeg_avcodec_close (ffmpegenc->context);
-    if (avcodec_get_context_defaults3 (ffmpegenc->context,
-            oclass->in_plugin) < 0)
-      GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
-    if (ffmpegenc->context->stats_in)
-      g_free (ffmpegenc->context->stats_in);
     GST_DEBUG_OBJECT (ffmpegenc, "avenc_%s: Failed to open libav codec",
         oclass->in_plugin->name);
-    return FALSE;
+    goto close_codec;
   }
 
 pix_fmt_err:
   {
-    gst_ffmpeg_avcodec_close (ffmpegenc->context);
-    if (avcodec_get_context_defaults3 (ffmpegenc->context,
-            oclass->in_plugin) < 0)
-      GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
     GST_DEBUG_OBJECT (ffmpegenc,
         "avenc_%s: AV wants different colourspace (%d given, %d wanted)",
         oclass->in_plugin->name, pix_fmt, ffmpegenc->context->pix_fmt);
-    return FALSE;
+    goto close_codec;
   }
 
 bad_input_fmt:
   {
     GST_DEBUG_OBJECT (ffmpegenc, "avenc_%s: Failed to determine input format",
         oclass->in_plugin->name);
-    return FALSE;
+    goto close_codec;
   }
-
-unsupported_codec:
+close_codec:
   {
     gst_ffmpeg_avcodec_close (ffmpegenc->context);
     if (avcodec_get_context_defaults3 (ffmpegenc->context,
             oclass->in_plugin) < 0)
       GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
-    GST_DEBUG ("Unsupported codec - no caps found");
+    goto cleanup_stats_in;
+  }
+cleanup_stats_in:
+  {
+    if (ffmpegenc->context->stats_in)
+      g_free (ffmpegenc->context->stats_in);
     return FALSE;
   }
 }
@@ -543,22 +574,10 @@ gst_ffmpegvidenc_propose_allocation (GstVideoEncoder * encoder,
 }
 
 static void
-ffmpegenc_setup_working_buf (GstFFMpegVidEnc * ffmpegenc)
+gst_ffmpegvidenc_free_avpacket (gpointer pkt)
 {
-  guint wanted_size =
-      ffmpegenc->context->width * ffmpegenc->context->height * 6 +
-      FF_MIN_BUFFER_SIZE;
-
-  /* Above is the buffer size used by ffmpeg/ffmpeg.c */
-
-  if (ffmpegenc->working_buf == NULL ||
-      ffmpegenc->working_buf_size != wanted_size) {
-    if (ffmpegenc->working_buf)
-      g_free (ffmpegenc->working_buf);
-    ffmpegenc->working_buf_size = wanted_size;
-    ffmpegenc->working_buf = g_malloc (ffmpegenc->working_buf_size);
-  }
-  ffmpegenc->buffer_size = wanted_size;
+  av_packet_unref ((AVPacket *) pkt);
+  g_slice_free (AVPacket, pkt);
 }
 
 static GstFlowReturn
@@ -567,9 +586,11 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
 {
   GstFFMpegVidEnc *ffmpegenc = (GstFFMpegVidEnc *) encoder;
   GstBuffer *outbuf;
-  gint ret_size = 0, c;
+  gint ret = 0, c;
   GstVideoInfo *info = &ffmpegenc->input_state->info;
   GstVideoFrame vframe;
+  AVPacket *pkt;
+  int have_data = 0;
 
   if (GST_VIDEO_CODEC_FRAME_IS_FORCE_KEYFRAME (frame))
     ffmpegenc->picture->pict_type = AV_PICTURE_TYPE_I;
@@ -595,18 +616,23 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
       gst_ffmpeg_time_gst_to_ff (frame->pts /
       ffmpegenc->context->ticks_per_frame, ffmpegenc->context->time_base);
 
-  ffmpegenc_setup_working_buf (ffmpegenc);
+  have_data = 0;
+  pkt = g_slice_new0 (AVPacket);
 
-  ret_size = avcodec_encode_video (ffmpegenc->context,
-      ffmpegenc->working_buf, ffmpegenc->working_buf_size, ffmpegenc->picture);
+  ret =
+      avcodec_encode_video2 (ffmpegenc->context, pkt, ffmpegenc->picture,
+      &have_data);
 
   gst_video_frame_unmap (&vframe);
 
-  if (ret_size < 0)
+  if (ret < 0 || !have_data)
+    g_slice_free (AVPacket, pkt);
+
+  if (ret < 0)
     goto encode_fail;
 
   /* Encoder needs more data */
-  if (!ret_size)
+  if (!have_data)
     return GST_FLOW_OK;
 
   /* save stats info if there is some as well as a stats file */
@@ -621,15 +647,10 @@ gst_ffmpegvidenc_handle_frame (GstVideoEncoder * encoder,
   /* Get oldest frame */
   frame = gst_video_encoder_get_oldest_frame (encoder);
 
-  /* Allocate output buffer */
-  if (gst_video_encoder_allocate_output_frame (encoder, frame,
-          ret_size) != GST_FLOW_OK) {
-    gst_video_codec_frame_unref (frame);
-    goto alloc_fail;
-  }
-
-  outbuf = frame->output_buffer;
-  gst_buffer_fill (outbuf, 0, ffmpegenc->working_buf, ret_size);
+  outbuf =
+      gst_buffer_new_wrapped_full (0, pkt->data, pkt->size, 0, pkt->size, pkt,
+      gst_ffmpegvidenc_free_avpacket);
+  frame->output_buffer = outbuf;
 
   /* buggy codec may not set coded_frame */
   if (ffmpegenc->context->coded_frame) {
@@ -653,17 +674,8 @@ encode_fail:
     GST_ERROR_OBJECT (ffmpegenc,
         "avenc_%s: failed to encode buffer", oclass->in_plugin->name);
 #endif /* GST_DISABLE_GST_DEBUG */
-    return GST_FLOW_OK;
-  }
-alloc_fail:
-  {
-#ifndef GST_DISABLE_GST_DEBUG
-    GstFFMpegVidEncClass *oclass =
-        (GstFFMpegVidEncClass *) (G_OBJECT_GET_CLASS (ffmpegenc));
-    GST_ERROR_OBJECT (ffmpegenc,
-        "avenc_%s: failed to allocate buffer", oclass->in_plugin->name);
-#endif /* GST_DISABLE_GST_DEBUG */
-    return GST_FLOW_ERROR;
+    /* avoid frame (and ts etc) piling up */
+    return gst_video_encoder_finish_frame (encoder, frame);
   }
 }
 
@@ -673,7 +685,9 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
   GstVideoCodecFrame *frame;
   GstFlowReturn flow_ret = GST_FLOW_OK;
   GstBuffer *outbuf;
-  gint ret_size;
+  gint ret;
+  AVPacket pkt;
+  int have_data = 0;
 
   GST_DEBUG_OBJECT (ffmpegenc, "flushing buffers with sending %d", send);
 
@@ -683,13 +697,12 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
 
   while ((frame =
           gst_video_encoder_get_oldest_frame (GST_VIDEO_ENCODER (ffmpegenc)))) {
+    memset (&pkt, 0, sizeof (pkt));
+    have_data = 0;
 
-    ffmpegenc_setup_working_buf (ffmpegenc);
+    ret = avcodec_encode_video2 (ffmpegenc->context, &pkt, NULL, &have_data);
 
-    ret_size = avcodec_encode_video (ffmpegenc->context,
-        ffmpegenc->working_buf, ffmpegenc->working_buf_size, NULL);
-
-    if (ret_size < 0) {         /* there should be something, notify and give up */
+    if (ret < 0) {              /* there should be something, notify and give up */
 #ifndef GST_DISABLE_GST_DEBUG
       GstFFMpegVidEncClass *oclass =
           (GstFFMpegVidEncClass *) (G_OBJECT_GET_CLASS (ffmpegenc));
@@ -707,20 +720,10 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
             (("Could not write to file \"%s\"."), ffmpegenc->filename),
             GST_ERROR_SYSTEM);
 
-    if (send) {
-      if (gst_video_encoder_allocate_output_frame (GST_VIDEO_ENCODER
-              (ffmpegenc), frame, ret_size) != GST_FLOW_OK) {
-#ifndef GST_DISABLE_GST_DEBUG
-        GstFFMpegVidEncClass *oclass =
-            (GstFFMpegVidEncClass *) (G_OBJECT_GET_CLASS (ffmpegenc));
-        GST_WARNING_OBJECT (ffmpegenc,
-            "avenc_%s: failed to allocate buffer", oclass->in_plugin->name);
-#endif /* GST_DISABLE_GST_DEBUG */
-        gst_video_codec_frame_unref (frame);
-        break;
-      }
-      outbuf = frame->output_buffer;
-      gst_buffer_fill (outbuf, 0, ffmpegenc->working_buf, ret_size);
+    if (send && have_data) {
+      outbuf = gst_buffer_new_wrapped_full (0, pkt.data, pkt.size, 0, pkt.size,
+          pkt.data, av_free);
+      frame->output_buffer = outbuf;
 
       if (ffmpegenc->context->coded_frame->key_frame)
         GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
@@ -728,7 +731,8 @@ gst_ffmpegvidenc_flush_buffers (GstFFMpegVidEnc * ffmpegenc, gboolean send)
       flow_ret =
           gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
     } else {
-      gst_video_codec_frame_unref (frame);
+      /* no frame attached, so will be skipped and removed from frame list */
+      gst_video_encoder_finish_frame (GST_VIDEO_ENCODER (ffmpegenc), frame);
     }
   }
 
@@ -772,6 +776,9 @@ gst_ffmpegvidenc_set_property (GObject * object,
     case PROP_COMPLIANCE:
       ffmpegenc->compliance = g_value_get_enum (value);
       break;
+    case PROP_MAX_THREADS:
+      ffmpegenc->max_threads = g_value_get_int (value);
+      break;
     default:
       if (!gst_ffmpeg_cfg_set_property (object, value, pspec))
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -808,6 +815,9 @@ gst_ffmpegvidenc_get_property (GObject * object,
     case PROP_COMPLIANCE:
       g_value_set_enum (value, ffmpegenc->compliance);
       break;
+    case PROP_MAX_THREADS:
+      g_value_set_int (value, ffmpegenc->max_threads);
+      break;
     default:
       if (!gst_ffmpeg_cfg_get_property (object, value, pspec))
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -834,6 +844,7 @@ gst_ffmpegvidenc_start (GstVideoEncoder * encoder)
       (GstFFMpegVidEncClass *) G_OBJECT_GET_CLASS (ffmpegenc);
 
   /* close old session */
+  gst_ffmpeg_avcodec_close (ffmpegenc->context);
   if (avcodec_get_context_defaults3 (ffmpegenc->context, oclass->in_plugin) < 0) {
     GST_DEBUG_OBJECT (ffmpegenc, "Failed to set context defaults");
     return FALSE;
@@ -855,10 +866,6 @@ gst_ffmpegvidenc_stop (GstVideoEncoder * encoder)
     fclose (ffmpegenc->file);
     ffmpegenc->file = NULL;
   }
-  if (ffmpegenc->working_buf) {
-    g_free (ffmpegenc->working_buf);
-    ffmpegenc->working_buf = NULL;
-  }
   if (ffmpegenc->input_state) {
     gst_video_codec_state_unref (ffmpegenc->input_state);
     ffmpegenc->input_state = NULL;
